#include "includes/Boot.hpp"

#ifdef DEBUG
  #include "includes/Screen.hpp"
#endif

#include "../includes/types.hpp"
//#include "../includes/asm.hpp"
#include "includes/GlobalDescriptorTable.hpp"
#include "includes/InterruptDescriptorTable.hpp"
#include "includes/ProgrammableInterruptController.hpp"

namespace BootProcess
{
  #ifdef DEBUG

    void PrintOK()
    {
      s8 oldAttributes = Screen::Attributes;
      Screen::Attributes = (Screen::Attributes & 0xF0) | 0x0A;
  
      Screen::PrintXY((s8*)"[OK]", 79 - 6, Screen::Y);
      Screen::Print((s8*)"\r\n");
  
      Screen::Attributes = oldAttributes;
    }

  #endif

  void Boot()
  {
    #ifdef DEBUG
      Screen::Print((s8*)"Loading GDT ...");
    #endif

    GlobalDescriptorTable::Init();

    #ifdef DEBUG
      PrintOK();
    #endif

    #ifdef DEBUG
      Screen::Print((s8*)"Loading IDT ...");
    #endif

    InterruptDescriptorTable::Init();

    #ifdef DEBUG
      PrintOK();
    #endif

    #ifdef DEBUG
      Screen::Print((s8*)"Configure PIC ...");
    #endif

    ProgrammableInterruptController::Init();

    #ifdef DEBUG
      PrintOK();
    #endif

  }
}
