#include "includes/GlobalDescriptorTable.hpp"
#include "../includes/types.hpp"
#include "../includes/lib.hpp"

namespace BootProcess
{
  namespace GlobalDescriptorTable
  {
    struct Descriptor KernelDescriptors[0xFF];
    struct Register KernelRegister;
    struct TaskStateSegment DefaultTaskStateSegment;

    void INIGDTDescriptor(u32 base, u32 limit, u8 access, u8 other, struct Descriptor* desc)
    {
      desc->Limit_0_15 = (u16)(limit & 0x0000FFFF);
      desc->Base_0_15 = (u16)(base & 0x0000FFFF);
      desc->Base_16_23 = (u8)((base & 0x00FF0000) >> 16);
      
      desc->Access.Accessed = access & 0x01;
      desc->Access.ReadWrite = (access & 0x02) >> 1;
      desc->Access.Direction_Conforming = (access & 0x04) >> 2;
      desc->Access.Executable = (access & 0x08) >> 3;
      desc->Access.AlwaysOne = (access & 0x10) >> 4;
      desc->Access.Privileges = (access & 0x60) >> 5;
      desc->Access.Present = (access & 0x80) >> 7;
  
      desc->Limit_16_19 = (limit & 0x000F0000) >> 16;
      desc->Flags = other & 0x0F;
      desc->Base_24_31 = (base & 0xFF000000) >> 24;
    }

    void Init()
    {
      DefaultTaskStateSegment.Reserved12 = 0x00;  //  Debug Flag ?
      DefaultTaskStateSegment.IOPBOffset = 0x00;  //  IOMap ?
      DefaultTaskStateSegment.esp0 = 0x1FFF0;
      DefaultTaskStateSegment.ss0 = 0x18;

    	// Initialisation des descripteurs de segment
    	INIGDTDescriptor(0x0, 0x0, 0x0, 0x0, &KernelDescriptors[0]);
    	INIGDTDescriptor(0x0, 0xFFFFF, 0x9B, 0x0D, &KernelDescriptors[1]);   // Code
    	INIGDTDescriptor(0x0, 0xFFFFF, 0x93, 0x0D, &KernelDescriptors[2]);   // Data
    	INIGDTDescriptor(0x0, 0x0, 0x97, 0x0D, &KernelDescriptors[3]);		   // Stack
    
    	INIGDTDescriptor(0x0, 0xFFFFF, 0xFF, 0x0D, &KernelDescriptors[4]);   // UCode
    	INIGDTDescriptor(0x0, 0xFFFFF, 0xF3, 0x0D, &KernelDescriptors[5]);	 // UData
    	INIGDTDescriptor(0x0, 0x0, 0xF7, 0x0D, &KernelDescriptors[6]);		   // UStack
    
    	INIGDTDescriptor((u32)&DefaultTaskStateSegment, 0x67, 0xE9, 0x00, &KernelDescriptors[7]);	// Descripteur de TSS

      KernelRegister.Base = 0x00000800;
      KernelRegister.Limit = 0xFF * 8;

      //  GDT Copy
      Lib::MemoryCopy((s8*)KernelDescriptors, (s8*)KernelRegister.Base, KernelRegister.Limit);

      //  Loading GDT Registers
      s8* pKernelRegister = (s8*)&KernelRegister;
      asm("lgdtl %0" :: "m"(*pKernelRegister));

      //  Segments init
    	asm("movw $0x10, %ax	  \n \
           movw %ax, %ds	    \n \
           movw %ax, %es	    \n \
           movw %ax, %fs	    \n \
           movw %ax, %gs	    \n \
           ljmp $0x08, $next	\n \
           next:		          \n");

    	asm("movw $0x18, %%ax   \n \
           movw %%ax, %%ss    \n \
           movl %0, %%esp"::"i" (0x0009FFF0));  //  KERNSTACK
    }
  }
}
