#include "includes/InterruptDescriptorTable.hpp"
#include "../includes/types.hpp"
#include "ASM/includes/Interrupts.hpp"
#include "../includes/lib.hpp"

namespace BootProcess
{
  namespace InterruptDescriptorTable
  {
    struct Descriptor KernelDescriptors[0xFF];
    struct Register KernelRegister;

    const s32 INTGATE = 0x8E00;
    const s32 TRAPGATE = 0xEF00;

    const s32 LIMIT = 0xFF;
    const s32 BASE = 0x00000000;

    // 'InterruptDescriptorTable' initialise un descripteur de segment situe en idt.
    // 'desc' est l'adresse lineaire du descripteur a initialiser.
    // L'argument 'type' doit avoir pour valeur INTGATE, TRAPGATE
    // ou TASKGATE.
    void InitDescriptor(u16 select, u32 offset, u16 type, struct Descriptor *desc)
    {
    	desc->Offset_0_15 = (offset & 0xffff);
    	desc->Select = select;
    	desc->Type = type;
    	desc->Offset_16_31 = (offset & 0xffff0000) >> 16;
    
    	return;
    }

    // Cette fonction initialise la IDT apres que le kernel soit charge 
    // en memoire. 
    void Init(void)
    {
    	// Initialisation des descripteurs systeme par defaut
    	for (int i = 0; i < LIMIT; i++) 
    		InitDescriptor(0x08, (u32)ASM_DefaultInt, INTGATE, &KernelDescriptors[i]);

    	// Les vecteurs 0 -> 31 sont reserves pour les exceptions
    	InitDescriptor(0x08, (u32)ASM_ExecGP, INTGATE, &KernelDescriptors[13]);     // #GP
    	InitDescriptor(0x08, (u32)ASM_ExecPF, INTGATE, &KernelDescriptors[14]);     // #PF
    
    	// Vecteurs d'interruption
    	InitDescriptor(0x08, (u32)ASM_IRQ0, INTGATE, &KernelDescriptors[32]);   //  Horloge
    	InitDescriptor(0x08, (u32)ASM_IRQ1, INTGATE, &KernelDescriptors[33]);   //  Clavier

    	// Appels systeme - int 0x30
    	InitDescriptor(0x08, (u32)ASM_SystemCalls, TRAPGATE, &KernelDescriptors[48]); 
    
    	// Initialisation de la structure pour IDTR
    	KernelRegister.Limit = LIMIT * 8;
    	KernelRegister.Base = BASE;
    
    	// Recopie de la IDT a son adresse
    	Lib::MemoryCopy((s8*)KernelDescriptors, (s8*)KernelRegister.Base, KernelRegister.Limit);
    
    	// Chargement du registre IDTR
  //  	asm("lidtl (kidtr)");
      char* pKernelRegister = (char*)&KernelRegister;
      asm("lidtl %0"::"m"(*pKernelRegister));
    }
  }
}
