#include "../../includes/types.hpp"

namespace BootProcess
{
  namespace GlobalDescriptorTable
  {
    #ifndef __KERNEL_BOOTPROCESS_INCLUDES_GLOBALDESCRIPTORTABLE_HPP__
    #define __KERNEL_BOOTPROCESS_INCLUDES_GLOBALDESCRIPTORTABLE_HPP__

      extern struct Descriptor KernelDescriptors[];
      extern struct Register KernelRegister;
      extern struct TaskStateSegment DefaultTaskStateSegment;

      //  http://wiki.osdev.org/GDT
      struct Descriptor
      {
        u16 Limit_0_15;
        u16 Base_0_15;
        u8  Base_16_23;
    
        struct
        {
          u8  Accessed:1;               //  Just set 0. The CPU set this to 1 when the segment is accessed.
          u8  ReadWrite:1;              //  Readable bit for code selector. Whether read access for this segment is allowed.
                                        //    Write access is never allowed for code segments.
                                        //  Writable bit for data selector. Whether write access for this segment is allowed.
                                        //    Read access is always allowed for data segment.
          u8  Direction_Conforming:1;   //  Direction for data selector. Tells the directions: 0 the segment grows up,
                                        //    1: the segment grows down, ie. the offset has to be greater than the base.
                                        //  Conforming bit for code selectors. If 1, code in this segment can be executed
                                        //    from an equal or lower privilege level. For example, code in ring 3 can far-jump
                                        //    to conforming code in a ring 2 segment. The privileges bit represent the highest
                                        //    privilege level that is allowed to execute this segment. For example, code in ring 0
                                        //    cannot far-jump to a conforming code segment with privileges bit == 0x02, while code
                                        //    in ring 2 and 3 can. Note that the privilege level remains the same, ie. far-jump
                                        //    from ring 3 to a privilege == 0x02 segment remains in ring 3 after the jump.
                                        //    If 0, code in this segment can only be executed from the ring set in privileges bit.  
          u8  Executable:1;             //  If 1 code in the segment can be executed, ie a code selector. If 0 is a data selector.
          u8  AlwaysOne:1;
          u8  Privileges:2;             //  Contains the ring level: 0 = highest (kernel), 1 = lowest (user applications)
          u8  Present:1;                //  This must be 1 for all valid selectors
        }__attribute__((packed)) Access;
    
        u8  Limit_16_19:4;
        
        u8  Flags:4;
        /*struct
        {
          u8  AlwaysZero:2;
          u8  Size:1;                   //  If 0, the selector defines 16 bits protected mode.
                                        //    If 1, it defines 32 bits protected mode.
                                        //    You can have both 16 bit and 32 bit selector at once.
          u8  Granularity:1;            //  If 0, the limit is in B blocks (byte granularity).
                                        //    If 1, the limit is in 4 KiB blocks (page granularity).
        }__attribute__((packed)) Flags;*/
    
        u8  Base_24_31;
      }__attribute__((packed));
    
      struct Register
      {
        u16 Limit;
        u32 Base;
      }__attribute__((packed));
  
      //  http://wiki.osdev.org/Task_State_Segment  
      struct TaskStateSegment
      {
        u16 Link; //  PreviousTask
        u16 Reserved1;
      	u32 esp0;
      	u16 ss0;
        u16 Reserved2;
      	u32 esp1;
      	u16 ss1;
        u16 Reserved3;
      	u32 esp2;
      	u16 ss2;
        u16 Reserved4;
      	u32 cr3;
      	u32 eip;
        u32 eflags;
        u32 eax;
        u32 ecx;
        u32 edx;
        u32 ebx;
        u32 esp;
        u32 ebp;
        u32 esi;
        u32 edi;
      	u16 es;
        u16 Reserved5;
      	u16 cs;
        u16 Reserved6;
      	u16 ss;
        u16 Reserved7;
      	u16 ds;
        u16 Reserved8;
      	u16 fs;
        u16 Reserved9;
      	u16 gs;
        u16 Reserved10;
      	u16 LocalDescriptorTableRegister;  //  Selector
        u16 Reserved11;
      	//u16 DebugFlag;
        u16 Reserved12;
        //u16 IOMap;
        u16 IOPBOffset; //  IOMap
      }__attribute__((packed));
    
    #endif

    void Init();
    void INIGDTDescriptor(u32 base, u32 limit, u8 access, u8 other, struct Descriptor* desc);
  }
}
