#include "includes/InterruptServiceRoutines.hpp"
#include "includes/types.hpp"
#include "BootProcess/includes/Screen.hpp" //  DEBUG
#include "includes/asm.hpp"
//#include "BootProcess/ASM/includes/Interrupts.hpp"
#include "includes/Scheduler.hpp"
#include "includes/VirtualMemoryManager.hpp"
#include "includes/PhysicalMemoryManager.hpp"

#include "includes/Processes.hpp"

namespace InterruptServiceRoutines
{
  u32 ClockCount = 0;
}

extern "C"
{
  void ISR_DefaultInterrupt(void)
  {
  }
  
  void ISR_GPExec(void)
  {
    BootProcess::Screen::Print((s8*)"#GP\r\n");
  	asm("cli; hlt");
  }
  
  void ISR_PFExec(void)
  {
  	u32 faulting_addr;
  	u32 eip;
  
   	asm("movl 60(%%ebp), %%eax; \
      	 mov %%eax, %0;         \
  		   mov %%cr2, %%eax;      \
  		   mov %%eax, %1": "=m"(eip), "=m"(faulting_addr): );

    BootProcess::Screen::PrintFormat((s8*)"#PF: eip: %p => faulting address: %p\r\n", eip, faulting_addr);
    asm("cli; hlt");
  }

  void ISR_ClockInterrupt(void)
  {
    Scheduler::Schedule();
  }

  void ISR_KeyboardInterrupt(void)
  {
    u8 i;

  	do
    {
  		i = ASM::INB(0x64);
  	} while ((i & 0x01) == 0);
  
  	i = ASM::INB(0x60);

    BootProcess::Screen::Print((s8*)"A");
  }

  void ISR_SystemCalls(s32 sys_num)
  {
    BootProcess::Screen::PrintFormat((s8*)"SYSCALL: %d", sys_num);
  }
}
