#include "includes/Processes.hpp"
#include "includes/types.hpp"

#include "BootProcess/includes/Screen.hpp"  //  DEBUG

#include "includes/VirtualMemoryManager.hpp"
#include "includes/PhysicalMemoryManager.hpp"
#include "includes/lib.hpp"

namespace Processes
{
  struct Process List[128];
  struct Process* Current = 0;
  s32 Count = 0;

  s32 LoadTask(s8* function, u32 size)
  {
    int PID = 1;

    while(List[PID].State != 0 && PID < 128)
      PID++;

    if (List[PID].State != 0)
    {
      BootProcess::Screen::Print((s8*)"\r\nPANIC: Not enough slot for process.\r\n");
      return 0;
    }

    VirtualMemoryManager::PageDirectory* pageDirectory = VirtualMemoryManager::NewPageDirectory(1);
    VirtualMemoryManager::InitUserPageDirectory(pageDirectory);

    VirtualMemoryManager::PageTable* pageTable = VirtualMemoryManager::NewPageTable(1);
    VirtualMemoryManager::AddPageTable(pageDirectory, pageTable, 0x40000000);

    u32 i = 0;
    u32 indexPageTable = 0;
    while(i < size)
    {
      u32 virtualAddress = 0x40000000 + i;
      u32 physicalAddress = PhysicalMemoryManager::GetFirstPageFrameAvailable();
      PhysicalMemoryManager::SetPageFrameUsed(physicalAddress);
      VirtualMemoryManager::AddPhysicalAddress(pageTable, virtualAddress, physicalAddress);

      Lib::MemoryCopy(function, (s8*)physicalAddress, i, 4096);

      indexPageTable++;      
      i += 4096;
    }

    //  User stack
    VirtualMemoryManager::PageTable* userStackPageTable = VirtualMemoryManager::NewPageTable(1);
    VirtualMemoryManager::AddPageTable(pageDirectory, pageTable, 0xE0000000);
    u32 physicalAddress2 = PhysicalMemoryManager::GetFirstPageFrameAvailable();
    PhysicalMemoryManager::SetPageFrameUsed(physicalAddress2);
    VirtualMemoryManager::AddPhysicalAddress(userStackPageTable, 0xE0000000, physicalAddress2);

    //  Kernel stack
    u32 physicalAddressKernelStack = PhysicalMemoryManager::GetFirstPageFrameAvailable();
    PhysicalMemoryManager::SetPageFrameUsed(physicalAddressKernelStack);

    Count++;

    List[PID].ID = PID;

    List[PID].Registers.ss = 0x33;
    List[PID].Registers.esp = 0xE0000000 + 4096 - 16;
    List[PID].Registers.eflags = 0x00;
    List[PID].Registers.cs = 0x23;
    List[PID].Registers.eip = 0x40000000;
    List[PID].Registers.ds = 0x2B;
    List[PID].Registers.es = 0x2B;
    List[PID].Registers.fs = 0x2B;
    List[PID].Registers.gs = 0x2B;

    List[PID].Registers.cr3 = (u32)pageDirectory;

    List[PID].KernelStack.ss0 = 0x18;
    List[PID].KernelStack.esp0 = physicalAddressKernelStack + 4096 - 16;  //  Current

    List[PID].Registers.eax = 0;
    List[PID].Registers.ecx = 0;
    List[PID].Registers.edx = 0;
    List[PID].Registers.ebx = 0;

    List[PID].Registers.ebp = 0;
    List[PID].Registers.esi = 0;
    List[PID].Registers.edi = 0;

    List[PID].PageDirectory = pageDirectory;

    List[PID].State = 1;

    return PID;
  }

  /*s32 LoadTask(s8* function, u32 size)
  {
    int PID = 1;

    while(List[PID].State != 0 && PID < 128)
      PID++;

    if (List[PID].State != 0)
    {
      BootProcess::Screen::Print((s8*)"\r\nPANIC: Not enough slot for process.\r\n");
      return 0;
    }

    int userMode = 1;

    struct VirtualMemoryManager::PageDirectory* pageDirectory = VirtualMemoryManager::NewPageDirectory(userMode);

    for (int j = 0; j < 256; j++)
      pageDirectory->Entries[j] = VirtualMemoryManager::KernelPageDirectory->Entries[j];

    for (int j = 256; j < 1024; j++)
    {
      pageDirectory->Entries[j].Present = 0;
      pageDirectory->Entries[j].PhysicalAddress = 0x00000;
    }

    struct VirtualMemoryManager::PageTable* pageTable = VirtualMemoryManager::NewPageTable(userMode);
    for (int j = 0; j < 1024; j++)
    {
      pageTable->Entries[j].Present = 0;
      pageTable->Entries[j].PhysicalAddress = 0x00000;
    }
//    BootProcess::Screen::PrintFormat((s8*)"PT: %x\n", virtualAddress);
    u32 virtualAddress = 0x40000000;  //  1 Go
    u32 pd3Index = virtualAddress >> 22;
    //u32 pt3Index = (virtualAddress >> 12) & 0x03FF;

    pageDirectory->Entries[pd3Index].Present = 1;
    pageDirectory->Entries[pd3Index].PhysicalAddress = ((u32)pageTable) >> 12;



    u32 i = 0;
    while (i < size)
    {
      u32 virtualAddress = 0x40000000 + i;  //  1 Go + offset
      
      u32 physicalAddress = PhysicalMemoryManager::GetFirstPageFrameAvailable();
      PhysicalMemoryManager::SetPageFrameUsed(physicalAddress);

//      VirtualMemoryManager::AddPageTable(pageDirectory, pageTable, virtualAddress, physicalAddress);
      
      u32 pt2Index = (virtualAddress >> 12) & 0x03FF;
      pageTable->Entries[pt2Index].Present = 1;
      pageTable->Entries[pt2Index].PhysicalAddress = physicalAddress >> 12;



      Lib::MemoryCopy(function, (s8*)physicalAddress, i, 4096);

      i += 4096;
    }

    //asm("cli; hlt");  //  DEBUG

    struct VirtualMemoryManager::PageTable* pageTableUserStack = VirtualMemoryManager::NewPageTable(userMode);
    u32 physicalAddress3 = PhysicalMemoryManager::GetFirstPageFrameAvailable();
    PhysicalMemoryManager::SetPageFrameUsed(physicalAddress3);
    
    u32 pdIndex = 0xE0000000 >> 22;
    u32 ptIndex = (0xE0000000 >> 12) & 0x03FF;
    pageDirectory->Entries[pdIndex].Present = 1;
    pageDirectory->Entries[pdIndex].PhysicalAddress = ((u32)pageTableUserStack >> 12);

    pageTableUserStack->Entries[ptIndex].Present = 1;
    pageTableUserStack->Entries[ptIndex].PhysicalAddress = physicalAddress3 >> 12;
    //VirtualMemoryManager::AddPageTable(pageDirectory, pageTableUserStack, 0xE0000000, physicalAddress3);  //  (u32)pageTableUserStack

    struct VirtualMemoryManager::PageTable* KernelStack = (struct VirtualMemoryManager::PageTable*)VirtualMemoryManager::NewPageTable(0);
    u32 physicalAddress2 = PhysicalMemoryManager::GetFirstPageFrameAvailable();
    PhysicalMemoryManager::SetPageFrameUsed(physicalAddress2);

    //VirtualMemoryManager::AddPageTable(pageDirectory, KernelStack, (u32)KernelStack, physicalAddress2);    
    // Assume physicalAddress == virtualAddress in kernel space    

    Count++;

    List[PID].ID = PID;

    List[PID].Registers.ss = 0x33;
    List[PID].Registers.esp = 0xE0000000 + 4096 - 16;
    List[PID].Registers.eflags = 0x00;
    List[PID].Registers.cs = 0x23;
    List[PID].Registers.eip = 0x40000000;
    List[PID].Registers.ds = 0x2B;
    List[PID].Registers.es = 0x2B;
    List[PID].Registers.fs = 0x2B;
    List[PID].Registers.gs = 0x2B;

    List[PID].Registers.cr3 = (u32)pageDirectory;

    List[PID].KernelStack.ss0 = 0x18;
    List[PID].KernelStack.esp0 = (u32)KernelStack + 4096 - 16;  //  Current

    List[PID].Registers.eax = 0;
    List[PID].Registers.ecx = 0;
    List[PID].Registers.edx = 0;
    List[PID].Registers.ebx = 0;

    List[PID].Registers.ebp = 0;
    List[PID].Registers.esi = 0;
    List[PID].Registers.edi = 0;

    List[PID].PageDirectory = pageDirectory;

    List[PID].State = 1;

    //asm volatile("mov %0, %%eax; mov %%eax; %%cr3" :: "m"()
    return PID;
  }*/
}
