#include "includes/Scheduler.hpp"
#include "includes/types.hpp"
#include "includes/Processes.hpp"
#include "BootProcess/includes/GlobalDescriptorTable.hpp"

#include "BootProcess/includes/Screen.hpp"    //  DEBUG

namespace Scheduler
{
  void Schedule()
  {
    u32* stackPointer;
    asm volatile("mov (%%ebp), %%eax; mov %%eax, %0" : "=m"(stackPointer));

    if (!Processes::Count)
      return;
    else if (Processes::Count == 1 && Processes::Current->ID != 0)
      return;

    Processes::Current->Registers.eflags = stackPointer[16];
    Processes::Current->Registers.cs = stackPointer[15];
    Processes::Current->Registers.eip = stackPointer[14];
    Processes::Current->Registers.eax = stackPointer[13];
    Processes::Current->Registers.ecx = stackPointer[12];
    Processes::Current->Registers.edx = stackPointer[11];
    Processes::Current->Registers.ebx = stackPointer[10];
    Processes::Current->Registers.ebp = stackPointer[8];
    Processes::Current->Registers.esi = stackPointer[7];
    Processes::Current->Registers.edi = stackPointer[6];
    Processes::Current->Registers.ds = stackPointer[5];
    Processes::Current->Registers.es = stackPointer[4];
    Processes::Current->Registers.fs = stackPointer[3];
    Processes::Current->Registers.gs = stackPointer[2];

    if (Processes::Current->Registers.cs != 0x08)
    {
      Processes::Current->Registers.esp = stackPointer[17];
      Processes::Current->Registers.ss = stackPointer[18];
    }
    else
    {
      Processes::Current->Registers.esp = stackPointer[9] + 12;
      Processes::Current->Registers.ss = BootProcess::GlobalDescriptorTable::DefaultTaskStateSegment.ss0;
    }

    Processes::Current->KernelStack.ss0 = BootProcess::GlobalDescriptorTable::DefaultTaskStateSegment.ss0;
    Processes::Current->KernelStack.esp0 = BootProcess::GlobalDescriptorTable::DefaultTaskStateSegment.esp0;

    s32 newPID = 0;
    for (s32 i = Processes::Current->ID + 1; i < 128 && newPID == 0; i++)
      if (Processes::List[i].State == 1)
        newPID = i;

    if (!newPID)
      for (s32 i = 1; i < Processes::Current->ID && newPID == 0; i++)
        if (Processes::List[i].State == 1)
          newPID = i;

    Processes::Process* p = &Processes::List[newPID];

    if (p->Registers.cs != 0x08)
      SwitchToTask(p->ID, 1);
    else
      SwitchToTask(p->ID, 0);
  }

  void SwitchToTask(int n, int mode)
  {
    Processes::Current = &Processes::List[n];

    BootProcess::GlobalDescriptorTable::DefaultTaskStateSegment.ss0 = Processes::Current->KernelStack.ss0;
    BootProcess::GlobalDescriptorTable::DefaultTaskStateSegment.esp0 = Processes::Current->KernelStack.esp0;

    u16 ss = Processes::Current->Registers.ss;
    u16 cs = Processes::Current->Registers.cs;
    u32 eflags = (Processes::Current->Registers.eflags | 0x200) & 0xFFFFBFFF;

    u16 kss;
    u32 kesp;

    if (mode == 1)
    {
      kss = Processes::Current->KernelStack.ss0;
      kesp = Processes::Current->KernelStack.esp0;
    }
    else
    {
      kss = Processes::Current->Registers.ss;
      kesp = Processes::Current->Registers.esp;
    }

  	asm("mov %0, %%ss; \
         mov %1, %%esp; \
         cmp %[KMODE], %[mode]; \
         je next; \
         push %2; \
         push %3; \
         next: \
         push %4; \
         push %5; \
         push %6; \
         push %7; \
         ljmp $0x08, $do_switch" 
         :: \
         "m"(kss), \
         "m"(kesp), \
         "m"(ss), \
         "m"(Processes::Current->Registers.esp), \
         "m"(eflags), \
         "m"(cs), \
         "m"(Processes::Current->Registers.eip), \
         "m"(Processes::Current), \
         [KMODE] "i"(0), \
         [mode] "g"(mode)
        );
  }
}
