#include "types.hpp"

namespace MultiBoot
{
  #ifndef __KERNEL_INCLUDES_MULTIBOOT_HPP__
  #define __KERNEL_INCLUDES_MULTIBOOT_HPP__

  struct Informations
  {
  	u32 Flags;

  	u32 MemoryLower;   // present if Flags[0] == 1
  	u32 MemoryHigher;  // present if Flags[0] == 1

    struct
    {
      u8 Partition3;  // 0xFF if unused
      u8 Partition2;  // 0xFF if unused
      u8 Partition1;  // 0xFF if unused
      u8 Drive;       // 0x00 for floppy, 0x80 for first hard disk
    }__attribute__((packed)) BootDevice;  // present if Flags[1] == 1
  
  	u32 CommandLine;   // present if Flags[2] == 1
  	u32 ModsCount;     // present if Flags[3] == 1
  	u32 ModsAddress;   // present if Flags[3] == 1
  
  	struct
    {
  		u32 NumTabSize;
  		u32 SizeStrSize;
  		u32 AddressAddress;
  		u32 ShndxReserved;
  	}__attribute__((packed)) Syms; // flags[4] xor Flags[5] == 1
    
    u32 MemoryMapLength;    // present if Flags[6] == 1
    u32 MemoryMapAddress;   // present if Flags[6] == 1
  
    u32 DrivesLength;     // present if Flags[7] == 1
    u32 DrivesAddress;    // present if Flags[7] == 1
  
    u32 ConfigurationTable;   // present if Flags[8] == 1
  
    u32 BootLoaderName;   // present if Flags[9] == 1
  
    u32 APMTable;         // present if Flags[10] == 1
  
    u32 VBEControlInfo;   // present if Flags[11] == 1
    u32 VBEModeInfo;
    u16 VBEMode;
    u16 VBEInterfaceSeg;
    u16 VBEInterfaceOff;
    u16 VBEInterfaceLength;
  };

  struct InformationsMod
  {
    u32 ModStart;
    u32 ModEnd;
    u32 String;
    u32 Reserved; // Always equals 0, set by boot loader
  };
  
  // -4: size
  /*struct multiboot_info_mmap
  {
    u32 BaseAddress;
    u32 Length;
    u32 Type;
  };*/
  
  struct InformationsDrive
  {
    u32 Size;
    u8  Number;
    u8  Mode;    // 0: CHS ; 1: LBA
    u16 Cylinders;
    u8  Heads;
    u8  Sectors;
    u16 Port1;
  };
  
  struct InformationsAPMTable
  {
    u16 Version;
    u16 CSeg;
    u32 Offset;
    u16 CSeg_16;
    u16 DSeg;
    u16 Flags;
    u16 CSegLength;
    u16 CSeg_16_Length;
    u16 Dseg_Length;
  };
  
  struct InformationsMemoryMap
  {
    u32 Size;
    u64 BaseAddress;
    u64 Length;
    u32 Type;   //  1: available, other values: used
  };

  #endif
}
