#include "types.hpp"

namespace VirtualMemoryManager
{
  #ifndef __KERNEL_INCLUDES_VIRTUAL_MEMORYMANAGER_HPP__
  #define __KERNEL_INCLUDES_VIRTUAL_MEMORYMANAGER_HPP__

    extern struct PageDirectory* KernelPageDirectory;
    extern struct PageDirectory* CurrentPageDirectory;

    /*struct PageDirectoryEntry
    {
      u32 Present: 1; //  Must be 1 to reference a page table
      u32 ReadWrite: 1; //  If 0, writes may not be allowed to the 4-MByte region controlled by this entry
      u32 UserSupervisor: 1;  //  If 0, user-mode accesses are not allowed to the 4-MByte region controlled by this entry
      u32 PageLevelWriteThrough: 1; //  Indirectly determines the memory type used to access the page table referenced by this entry
      u32 PageLevelCacheDisable: 1; //  Indirectly determines the memory type used to access the page table referenced by this entry
      u32 Accessed: 1;  //  Indicates whether this entry has been used for linear-address translation
      u32 Ignored: 1;
      u32 PageSize: 1;  //  If CR4.PSE = 1, must be 0 (otherwise, this entry maps a 4-MByte page; see Table 4-4); otherwise, ignored
      u32 Ignored2: 3;
      u32 PhysicalAddress: 20;  //  Physical address of 4-KByte aligned page table referenced by this entry
    }__attribute__((packed));*/
    struct PageDirectoryEntry
    {
      u32 Present: 1; //  Must be 1 to reference a page table
      u32 ReadWrite: 1; //  If 0, writes may not be allowed to the 4-MByte region controlled by this entry
      u32 UserSupervisor: 1;  //  If 0, user-mode accesses are not allowed to the 4-MByte region controlled by this entry
      u32 Accessed: 1;  //  Indicates whether this entry has been used for linear-address translation
      u32 Ignored: 1;
      u32 Ignored2: 7;
      u32 PhysicalAddress: 20;  //  Physical address of 4-KByte aligned page table referenced by this entry
    }__attribute__((packed));
  
    //  Manage 4 ko
    /*struct PageTableEntry
    {
      u32 Present: 1; //  Must be 1 to map a 4kb page
      u32 ReadWrite: 1; //  If 0, writes may not be allowed to the 4-KByte page referenced by this entry
      u32 UserSupervisor: 1;  //  If 0, user-mode accesses are not allowed to the 4-KByte page referenced by this entry
      u32 PageLevelWriteThrough: 1;  //  Indirectly determines the memory type used to access the 4-KByte page referenced by this entry
      u32 PageLevelCacheDisable: 1; //  Indirectly determines the memory type used to access the 4-KByte page referenced by this entry
      u32 Accessed: 1;  //  Indicates whether software has accessed the 4-KByte page referenced by this entry
      u32 Dirty: 1; //  Indicates whether software has written to the 4-KByte page referenced by this entry
      u32 PAT: 1; //  If the PAT is supported, indirectly determines the memory type used to access the 4-KByte page referenced by this
                  //    entry (see Section 4.9.2); otherwise, reserved (must be 0)
      u32 Global: 1;  //  If CR4.PGE = 1, determines whether the translation is global (see Section 4.10); ignored otherwise
      u32 Ignored: 3;
      u32 PhysicalAddress: 20;  //  Physical address of the 4-KByte page referenced by this entry
                                //    Frame Address
    }__attribute__((packed));*/
    struct PageTableEntry
    {
      u32 Present: 1; //  Must be 1 to map a 4kb page
      u32 ReadWrite: 1; //  If 0, writes may not be allowed to the 4-KByte page referenced by this entry
      u32 UserSupervisor: 1;  //  If 0, user-mode accesses are not allowed to the 4-KByte page referenced by this entry
      u32 Accessed: 1;  //  Indicates whether software has accessed the 4-KByte page referenced by this entry
      u32 Dirty: 1; //  Indicates whether software has written to the 4-KByte page referenced by this entry
      u32 Ignored: 7;
      u32 PhysicalAddress: 20;  //  Physical address of the 4-KByte page referenced by this entry
                                //    Frame Address
    }__attribute__((packed));
  
    // Manage 4 ko * 1024 = 4 Mo
    struct PageTable
    {
      struct PageTableEntry Entries[1024];
    }__attribute__((packed));
  
    struct PageDirectory
    {
      struct PageDirectoryEntry Entries[1024];
    }__attribute__((packed));

  #endif

  void Init();
  u32 GetRealAddress(u32 virtualAddress);
  struct PageDirectory* NewPageDirectory(u8 user);
  void ReleasePageDirectory(struct PageDirectory* pageDirectory);
  struct PageTable* NewPageTable(u8 user);
  void ReleasePageTable(struct PageTable* pageTable);
//  void AddPageTable(struct PageDirectory* pageDirectory, struct PageTable* pageTable, u32 virtualAddress, u32 physicalAddress);
  void AddPageTable(struct PageDirectory* pageDirectory, struct PageTable* pageTable, u32 virtualAddress);
  void InitPageTable(struct PageTable* pageTable, int user);
  void InitUserPageDirectory(PageDirectory* pageDirectory);
  void AddPhysicalAddress(PageTable* pageTable, u32 virtualAddress, u32 physicalAddress);
}
