#include "includes/main.hpp"
//#include "lib/includes/printk.hpp"

#include "BootProcess/includes/Screen.hpp"

#include "includes/types.hpp"
#include "BootProcess/includes/Boot.hpp"
#include "includes/asm.hpp"
#include "includes/MultiBoot.hpp"
#include "includes/PhysicalMemoryManager.hpp"
#include "includes/VirtualMemoryManager.hpp"

#include "includes/Processes.hpp"

void task1()
{
	char *msg = (char *) 0x40001000;

	msg[0] = 'T';
	msg[1] = 'a';
	msg[2] = 's';
	msg[3] = 'k';
	msg[4] = '1';
	msg[5] = '\n';
	msg[6] = 0x00;

	while (true)
  {
		asm("mov %0, %%ebx; mov $0x01, %%eax; int $0x30":: "m"(msg));
		for (int i = 0; i < 1000000; i++);  //  Wait a while
	}

	return;			/* Never go there */
}

void task2()
{
	char *msg = (char *) 0x40001000;

	msg[0] = 'T';
	msg[1] = 'a';
	msg[2] = 's';
	msg[3] = 'k';
	msg[4] = '1';
	msg[5] = '\n';
	msg[6] = 0x00;

	while (true)
  {
		asm("mov %0, %%ebx; mov $0x01, %%eax; int $0x30":: "m"(msg));
		for (int i = 0; i < 1000000; i++);  //  Wait a while
	}

	return;			/* Never go there */
}

extern "C"
{
  void kmain(MultiBoot::Informations* multiBootInformations)
  {
    BootProcess::Screen::Clear();

    #ifdef DEBUG
      BootProcess::Screen::Print((s8*)"*********************************\r\n");
      BootProcess::Screen::Print((s8*)"*  CAUTION: THIS IS DEBUG MODE  *\r\n");
      BootProcess::Screen::Print((s8*)"*********************************\r\n");
    #endif

    ASM::CLI();

    if ((multiBootInformations->Flags & 0x00000040) != 0x00000040)
    {
      // PANIC
      ASM::CLI();
      BootProcess::Screen::Print((s8*)"\r\nPANIC: Memory Map is empty!\r\n");
      ASM::HLT();
    }

    #ifdef DEBUG
      BootProcess::Screen::Print((s8*)"Init PMM ...");
    #endif

    PhysicalMemoryManager::Init(multiBootInformations);

    #ifdef DEBUG
      BootProcess::PrintOK();
    #endif    


    #ifdef DEBUG
      BootProcess::Screen::Print((s8*)"Init VMM ...");
    #endif

    VirtualMemoryManager::Init();

    #ifdef DEBUG
      BootProcess::PrintOK();
    #endif

    BootProcess::Boot();

    #ifdef DEBUG
      BootProcess::Screen::Print((s8*)"Loading Task Register ...");
    #endif

  	asm("movw $0x38, %ax; ltr %ax");

    #ifdef DEBUG
      BootProcess::PrintOK();
    #endif

    Processes::Current = &Processes::List[0];
    Processes::Current->ID = 0;
    Processes::Current->State = 1;
    Processes::Current->Registers.cr3 = (u32)VirtualMemoryManager::KernelPageDirectory;

    Processes::LoadTask((s8*)&task1, 0x2000);
    Processes::LoadTask((s8*)&task2, 0x2000);

    ASM::STI();

    #ifdef DEBUG
      BootProcess::Screen::Print((s8*)"System Ready ! Booting now ...\r\n");
    #endif

    while(true)
    {
      //ASM::STI();  
      ASM::HLT();
    }
  }
}
